// Includes
#include <Servo.h> 
#include "TimerThree.h"
// Constants
const int TurnAmount   =1275;
const int GridSize   =15;
const int StrightAmount=1900;
const int RobotWidth = 640;
int scan_one_cell=740;//lower number makes it see further
const float DegToRad = 0.0174532925;
const int numReadings =5;
// Sensor byte indices - offsets in packets 0, 5 and 6
#define SenBumpDrop 0            
#define SenWall 1
#define SenCliffL 2
#define SenCliffFL 3
#define SenCliffFR 4
#define SenCliffR 5

#define Sen6Size 52
#define SenChAvailable  39


// Global variables
int NoSteps=0;
int timer_cnt = 0;
int timer_on = 0;
int sensors[Sen6Size];
int distance = 0;
signed long int angle = 0;

// Scanning Variables
unsigned long int scan_angle=750;//position of scanner, in units of servo command
unsigned int tmp=255;
unsigned int object_found=0;

int old_state=1;//records current pointed direction of robot: 1 is up, 2 is right, 3 is down, and 4 is left (clockwise)
int new_state=1;//required direction of robot
//X is vertical, Y is horizontal
int Map[GridSize][GridSize];


Servo myservo;
// Wave Front Variables
int nothing=0;
int wall=255;
int goal=1;
int robot=254;

//starting robot/goal locations
int robot_x=8;
int robot_y=8;
int goal_x=2;
int goal_y=2;
int cell_size=330;//the size of the robot

//Map locations
int x=0;
int y=0;

//temp variables
int counter=0;

//when searching for a node with a lower value
int minimum_node=250;
int min_node_location=0;
int reset_min=250;//anything above this number is a special item, ie a wall or robot
// Global variables

int turn_angle = 0;
int turn_dir = 1;
int turning = 0;



void setup() 
{ 
  Serial.begin(9600);
  myservo.attach(8);  // attaches the servo on pin 9 to the servo object 
  Timer3.initialize();
  setupStepper();

  for (int H=0; H<GridSize;H++)
  {
    for (int W=0; W<GridSize;W++)
    {
      Map[H][W]=0;
    }
  }
  ////////////////////Enter your code below this line////////////////////
  //reset encoder function
  angle = 0;
  distance = 0;
  //store robot location in Map
  Map[robot_x][robot_y]=robot;
  //store robot location in Map
  Map[goal_x][goal_y]=goal;
  // Drive around until a button or unsafe condition is detected
}


void loop() 
{
  while(   (!sensors[SenCliffL])
    && (!sensors[SenCliffFL])
    && (!sensors[SenCliffFR])
    && (!sensors[SenCliffR])
    && (!sensors[SenChAvailable]) && Map[robot_x][robot_y]!=goal)//program ends when robot reaches goal
  {
    find_walls();
    new_state=propagate_wavefront();
    //////////////wavefront code//////////////
    while(new_state!=old_state && new_state!=0)
    {
      //if not pointed in the right direction, rotate
      if (abs(old_state - new_state) == 2)//rotate 180 degrees
      {
        rotate_CCW(180,250);
        rotate_CW(90,150);
      }
      else if ((signed int)(old_state - new_state) == -1 || (signed int)(old_state - new_state) == 3)//rotate 90 degrees CW
        rotate_CW(90,150);
      else if ((signed int)(old_state - new_state) == 1 || (signed int)(old_state - new_state) == -3)//rotate 90 degrees CCW
        rotate_CCW(90,150);
      //make new state the old state
      old_state=new_state;
      //recheck with new location
      find_walls();
      //find new location to go to
      new_state=propagate_wavefront();
    }
    //there is no solution
    //Serial.println(new_state);
    if (new_state==0)
    {
      delay(2500);
      //reset the entire Map
      clear_Map();
    }
    //go to new location
    else//move if there is a solution, otherwise dont move forward
    {
      if (new_state==1)
        robot_x--;
      if (new_state==2)
        robot_y++;
      if (new_state==3)
        robot_x++;
      if (new_state==4)
        robot_y--;
      
        straight(cell_size,100);//distance, velocity
      //update new location of robot
      
    }
    //make new state the old state
    if(new_state!=0)
      old_state=new_state;
  }
  if (Map[robot_x][robot_y]=goal)
  {
    if (goal_x=10)
      goal_x=2;
    else
      goal_x=10;

    Map[robot_x][robot_y]=robot;
    //store robot location in Map
    Map[goal_x][goal_y]=goal;    
  }
}



void rotate_CCW(signed long int angle_tmp, signed long int velocity)
{
  turnRight();
  Timer3.attachInterrupt(MoveTurn, 900);
  angle=0;
  while(NoSteps!=0)
  {
  }
}


//rotate a certain angle - 100 = 90 degrees!
void rotate_CW(signed long int angle_tmp, signed long int velocity)
{
  turnLeft();
  Timer3.attachInterrupt(MoveTurn, 900);
  angle=0;
  while(NoSteps!=0)
  {
  }

}


//drive straight function (use negative velocity to go in reverse)
void straight(signed long int distance_tmp, signed long int velocity)
{
  stright();
  Timer3.attachInterrupt(MoveStright, 900);
  
  //  Serial.println("A"); // send the map to the serial port, for the processing
    for (int H=0; H<GridSize;H++)
    {
      for (int W=0; W<GridSize;W++)
      {
        delay(9);
        Serial.println(Map[H][W]);
      }
    }
    while(NoSteps!=0)
  {
  }
}

